#ifndef _UTIL_H
#define _UTIL_H

#include <syslog.h>
#include <errno.h>

/* logging */
#ifdef __SYSLOG__

  #define LLOG(level, fmt, args...) \
    syslog(level, "[%s:%s:%d]  " fmt, __FILE__, __FUNCTION__, __LINE__, ##args)

  #define LOG(level, fmt, args...) \
    syslog(level, fmt, ##args)

  #define CERR(fmt, args...) \
    LLOG(LOG_ERR, fmt, ##args)

#else

  #define LLOG(level, fmt, args...) \
    printf("[%s:%s:%d]  " fmt, __FILE__, __FUNCTION__, __LINE__, ##args)

  #define LOG(level, fmt, args...) \
    printf(fmt, ##args)

  #define CERR(fmt, args...) \
    fprintf(stderr, "[%s:%s:%d]  " fmt, __FILE__, __FUNCTION__, __LINE__, ##args)

#endif

#define CERR_RET(retval, fmt, args...) \
  { CERR(fmt, ##args); return retval; }

#define ERR() \
  CERR("%s\n", strerror(errno))

#define ERR_RET(retval) \
  { ERR(); return retval; }

/* debug */
#ifdef __DEBUG__

  #define DB(fmt, args...) \
    LLOG(LOG_DEBUG, fmt, ##args)

#else

  #define DB(fmt, args...) {}

#endif

void bufdump(void *buf, int size);
inline int max(int a, int b);
inline int min(int a, int b);

#endif /* _UTIL_H */
