#ifndef _IPQEXTCTL_H
#define _IPQEXTCTL_H

#include <libipq.h>
#include <router/ipqext/ipqext.h>

/* unix domain socket pathname */
#define IPQEXTD_SOCKET_PATHNAME "/tmp/ipqextd.socket"
#define IPQEXT_SOCKET_PATHNAME "/tmp/ipqext.socket"

/* control command */
#define IPQEXTCTL_CMD_ADD        0x00000001
#define IPQEXTCTL_CMD_DEL        0x00000002
#define IPQEXTCTL_CMD_MOVE       0x00000004
#define IPQEXTCTL_CMD_START      0x00000008
#define IPQEXTCTL_CMD_STOP       0x00000010
#define IPQEXTCTL_CMD_GROUP      0x00000020
#define IPQEXTCTL_CMD_RESET      0x00000040
#define IPQEXTCTL_CMD_INFO       0x00000080
#define IPQEXTCTL_CMD_FLUSH      0x00000100
#define IPQEXTCTL_CMD_LIST       0x00000200
#define IPQEXTCTL_CMD_MARK       0x00000400
#define IPQEXTCTL_CMD_HELP       0x00000800

/* response code */
#define IPQEXTCTL_STAT_OK        0x00000000
#define IPQEXTCTL_STAT_CONT      0x00000001
#define IPQEXTCTL_STAT_FAIL      0xffffffff
#define IPQEXTCTL_STAT_UNKNOWN   0xfffffffe

/* ipqextctl request */
typedef struct {
  int command;
  char extname[IPQEXT_NAME_LEN + 1];
  char param[IPQEXT_PARAM_LEN + 1];
} ipqextctl_request_t;

/* ipqextctl response */
typedef struct {
  int status;
  char extname[IPQEXT_NAME_LEN];
  char param[IPQEXT_PARAM_LEN];
} ipqextctl_response_t;

#define IPQEXTCTL_INIT_UDP_SOCKET(pathname) \
  ipqextctl_init_socket(SOCK_DGRAM, pathname)

#define IPQEXTCTL_INIT_TCP_SOCKET(pathname) \
  ipqextctl_init_socket(SOCK_STREAM, pathname)

void ipqextctl_set_sockaddr(struct sockaddr_un *addr, char *pathname);
int ipqextctl_init_socket(int type, char *pathname);
void ipqextctl_close_socket(int sd, char *pathname);
void ipqextctl_show_request(ipqextctl_request_t *request);
void ipqextctl_show_response(ipqextctl_response_t *response);

#endif /* _IPQEXTCTL_H */
