// indicate this is an ipqext extension
#define _IPQEXT_EXTENSION_

//#define __DEBUG__

//----------------------------------------------------------------------//
//
//  ipqext extension module template
//
//----------------------------------------------------------------------//

#include <stdio.h>
#include <linux/netfilter.h>
#include <libipq.h>
#include <router/ipqext/ipqext.h>
#include <lib/util.h>

/*
 * define extension name here
 *
 * extension name must match with the output module name
 * ( i.e. ipqext_{extension name} )
 */
const char extension_name[] = "template";

//----------------------------------------------------------------------//
//
//  module functions
//

/*
 * ipqext structure
 */
static ipqext_t ext = {
  prev      : NULL,
  next      : NULL,
  handle    : NULL,
  active    : 0,
  name      : extension_name,
  version   : "1.0",
  groupmask : 0,
  init      : &init,
  fini      : &fini,
  proc      : &proc,
  start     : &start,
  stop      : &stop,
  reset     : &reset,
  info      : &info,
  help      : &help,
};

/*
 * packet processing function
 *
 * return: NF_ACCEPT   packet is accepted and leaves ipqext immediately
 *         NF_QUEUE    packet is queued for further ipqext processing
 *         NF_DROP     packet is explicitly dropped
 *
 *         negative verdict to indicate packet is altered
 */
int proc(ipq_packet_msg_t *m)
{
  unsigned int retval = NF_QUEUE;

  DB("%s: proc invoked\n", extension_name);
  return retval;
}

/*
 * extension start function
 *
 * return 0 on success, -1 or nonzero value on error
 */
int start()
{
  DB("%s: start invoked\n", extension_name);
  return 0;
}

/*
 * extension stop function
 *
 * return 0 on success, -1 or nonzero value on error
 */
int stop()
{
  DB("%s: stop invoked\n", extension_name);
  return 0;
}

/*
 * extension reset function
 *
 * return 0 on success, -1 or nonzero value on error
 */
int reset()
{
  DB("%s: reset invoked\n", extension_name);
  return 0;
}

/*
 * extension info function
 *
 * return a pointer to a char buffer or NULL
 */
char infobuf[] = " info here ";

const char *info()
{
  DB("%s: info invoked\n", extension_name);
  return infobuf;
}

/*
 * extension help function
 *
 * return a pointer to a char buffer or NULL
 */
const char helpbuf[] = " help here ";

const char *help()
{
  DB("%s: help invoked\n", extension_name);
  return helpbuf;
}

/*
 * extension initial pass function
 *
 * return 0 on success, -1 or nonzero value on error
 */
int init_pass(void *handle, char *param, int paramlen)
{
  DB("%s: init_pass invoked\n", extension_name);
  // set module handle
  ext.handle = handle;

  // parse parameter buffer

  return 0;
}

/*
 * extension initalization function
 *
 * return 0 on success, -1 or nonzero value on error
 */
int init()
{
  DB("%s: init invoked\n", extension_name);
  // initalization codes here

  // register function must always be the last call
  ipqext_register(&ext);

  return 0;
}

/*
 * extension finalization function
 *
 * return 0 on success, -1 or nonzero value on error
 */
int fini()
{
  DB("%s: fini invoked\n", extension_name);
  // unregister function must always be the first call
  ipqext_unregister(&ext);

  // finialization codes here

  return 0;
}

/*
 * module initalization function
 */
void _init(void)
{
  DB("%s: _init invoked\n", extension_name);
}

/*
 * module finalization function
 */
void _fini(void)
{
  DB("%s: _fini invoked\n", extension_name);
}

//----------------------------------------------------------------------//
